@@ -20,6 +20,7 @@ from debtcollector import removals
 import prettytable
 import six
 
+from automaton import _utils as utils
 from automaton import exceptions as excp
 
 
@@ -49,15 +50,18 @@ class FiniteMachine(object):
     entering a new state.
 
     NOTE(harlowja): reactions will *only* be called when the generator/iterator
-    from ``runner.run_iter()`` does *not* send back a new event (they will
-    always be called if the ``runner.run()`` method is used). This allows for
-    two unique ways (these ways can also be intermixed) to use this state
-    machine when using ``runner.run_iter()``; one where *external* events
-    trigger the next state transition and one where *internal* reaction
-    callbacks trigger the next state transition. The other way to use this
-    state machine is to skip using ``runner.run()`` or ``runner.run_iter()``
-    completely and use the ``process_event()`` method explicitly and trigger
-    the events via some *external* functionality/triggers...
+    from :py:meth:`~automaton.runners.Runner.run_iter` does *not* send
+    back a new event (they will always be called if the
+    :py:meth:`~automaton.runners.Runner.run` method is used). This allows
+    for two unique ways (these ways can also be intermixed) to use this state
+    machine when using :py:meth:`~automaton.runners.Runner.run`; one
+    where *external* event trigger the next state transition and one
+    where *internal* reaction callbacks trigger the next state
+    transition. The other way to use this
+    state machine is to skip using  :py:meth:`~automaton.runners.Runner.run`
+    or :py:meth:`~automaton.runners.Runner.run_iter`
+    completely and use the :meth:`.process_event` method explicitly and
+    trigger the events via some *external* functionality/triggers...
     """
 
     # Result of processing an event (cause and effect...)
@@ -140,6 +144,15 @@ class FiniteMachine(object):
         }
         self._transitions[state] = collections.OrderedDict()
 
+    def is_actionable_event(self, event):
+        """Check whether the event is actionable in the current state."""
+        current = self._current
+        if current is None:
+            return False
+        if event not in self._transitions[current.name]:
+            return False
+        return True
+
     def add_reaction(self, state, event, reaction, *args, **kwargs):
         """Adds a reaction that may get triggered by the given event & state.
 
@@ -185,6 +198,10 @@ class FiniteMachine(object):
             raise excp.NotFound("Can not add a transition on event '%s' that"
                                 " ends in a undefined state '%s'"
                                 % (event, end))
+        if self._states[start]['terminal']:
+            raise excp.InvalidState("Can not add a transition on event '%s'"
+                                    " that starts in the terminal state '%s'"
+                                    % (event, start))
         self._transitions[start][event] = _Jump(end,
                                                 self._states[end]['on_enter'],
                                                 self._states[start]['on_exit'])
@@ -321,22 +338,26 @@ class FiniteMachine(object):
                     target = self._transitions[state][event]
                     row = [pretty_state, event, target.name]
                     if target.on_enter is not None:
-                        try:
-                            row.append(target.on_enter.__name__)
-                        except AttributeError:
-                            row.append(target.on_enter)
+                        row.append(utils.get_callback_name(target.on_enter))
                     else:
                         row.append(empty)
                     if target.on_exit is not None:
-                        try:
-                            row.append(target.on_exit.__name__)
-                        except AttributeError:
-                            row.append(target.on_exit)
+                        row.append(utils.get_callback_name(target.on_exit))
                     else:
                         row.append(empty)
                     tbl.add_row(row)
             else:
-                tbl.add_row([pretty_state, empty, empty, empty, empty])
+                on_enter = self._states[state]['on_enter']
+                if on_enter is not None:
+                    on_enter = utils.get_callback_name(on_enter)
+                else:
+                    on_enter = empty
+                on_exit = self._states[state]['on_exit']
+                if on_exit is not None:
+                    on_exit = utils.get_callback_name(on_exit)
+                else:
+                    on_exit = empty
+                tbl.add_row([pretty_state, empty, empty, on_enter, on_exit])
         return tbl.get_string()
 
 
